/**
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

const Gio = imports.gi.Gio;
const Lang = imports.lang;

const DBusIface = <interface name="org.freedesktop.DBus">
<method name="GetNameOwner">
    <arg type="s" direction="in" />
    <arg type="s" direction="out" />
</method>
<method name="ListNames">
    <arg type="as" direction="out" />
</method>
<signal name="NameOwnerChanged">
    <arg type="s" direction="out" />
    <arg type="s" direction="out" />
    <arg type="s" direction="out" />
</signal>
</interface>;
const DBusProxy = Gio.DBusProxy.makeProxyWrapper(DBusIface);

const PropertiesIface = <interface name="org.freedesktop.DBus.Properties">
<method name="Get">
    <arg type="s" direction="in" />
    <arg type="s" direction="in" />
    <arg type="v" direction="out" />
</method>
<signal name="PropertiesChanged">
    <arg type="s" direction="out" />
    <arg type="a{sv}" direction="out" />
    <arg type="as" direction="out" />
</signal>
</interface>;
const PropertiesProxy = Gio.DBusProxy.makeProxyWrapper(PropertiesIface);

/**
 * Mostly taken from the output of
 * dbus-send --print-reply --dest=org.gnome.SettingsDaemon \
   /org/gnome/SettingsDaemon/MediaKeys org.freedesktop.DBus.Introspectable.Introspect
 */
const MediaKeysIface = <interface name="org.gnome.SettingsDaemon.MediaKeys">
  <annotation name="org.freedesktop.DBus.GLib.CSymbol" value="gsd_media_keys_manager">
  </annotation>
  <method name="GrabMediaPlayerKeys">
    <arg type="s" name="application" direction="in">
    </arg>
    <arg type="u" name="time" direction="in">
    </arg>
  </method>
  <method name="ReleaseMediaPlayerKeys">
    <arg type="s" name="application" direction="in">
    </arg>
  </method>
  <signal name="MediaPlayerKeyPressed">
    <arg type="s" name="application">
    </arg>
    <arg type="s" name="key">
    </arg>
  </signal>
</interface>;

const MediaKeysProxy = Gio.DBusProxy.makeProxyWrapper(MediaKeysIface);

const MediaServer2Iface = <interface name="org.mpris.MediaPlayer2">
<method name="Raise" />
<method name="Quit" />
<property name="CanRaise" type="b" access="read" />
<property name="CanQuit" type="b" access="read" />
<property name="Identity" type="s" access="read" />
<property name="DesktopEntry" type="s" access="read" />
</interface>;
const MediaServer2Proxy = Gio.DBusProxy.makeProxyWrapper(MediaServer2Iface);

const MediaServer2PlayerIface = <interface name="org.mpris.MediaPlayer2.Player">
<method name="PlayPause" />
<method name="Pause" />
<method name="Play" />
<method name="Stop" />
<method name="Next" />
<method name="Previous" />
<method name="SetPosition">
    <arg type="o" direction="in" />
    <arg type="x" direction="in" />
</method>
<property name="CanPause" type="b" access="read" />
<property name="CanSeek" type="b" access="read" />
<property name="Metadata" type="a{sv}" access="read" />
<property name="Volume" type="d" access="readwrite" />
<property name="PlaybackStatus" type="s" access="read" />
<property name="Position" type="x" access="read" />
<signal name="Seeked">
    <arg type="x" direction="out" />
</signal>
</interface>
const MediaServer2PlayerProxy = Gio.DBusProxy.makeProxyWrapper(MediaServer2PlayerIface);

function DBus() {
    return new DBusProxy(Gio.DBus.session, 'org.freedesktop.DBus',
                         '/org/freedesktop/DBus');
}

function MediaKeys() {
    return new MediaKeysProxy(Gio.DBus.session, 'org.gnome.SettingsDaemon',
                       '/org/gnome/SettingsDaemon/MediaKeys');
}

function MediaServer2Player(owner, callback) {
    new MediaServer2PlayerProxy(Gio.DBus.session, owner,
                                '/org/mpris/MediaPlayer2',
                                callback);
}