/**
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**/

const Main = imports.ui.main;
const Lang = imports.lang;
const Gio = imports.gi.Gio;
const Shell = imports.gi.Shell;
const Me = imports.misc.extensionUtils.getCurrentExtension();
const DBusIface = Me.imports.dbus

function MediaKeysClient() {
    this._init();
}


MediaKeysClient.prototype = {

    _init: function() {

    },

    _connectSignals: function() {
        global.log("Connecting Signals")
        this._proxy.GrabMediaPlayerKeysRemote("MediaKeystoMPRIS2",Lang.bind(this, this._onGrab));

        this._mediaKeyId = this._proxy.connectSignal('MediaPlayerKeyPressed',
            Lang.bind(this, this._onMediaKey));

    },

    _connectPlayer: function() {
      if (this._playerName != null){
        global.log("Connecting Player: " + this._playerName)
        // Player
        this._connectSignals();
        new DBusIface.MediaServer2Player(this._playerName,
                                         Lang.bind(this, function(proxy) {
                                             this._mediaServerPlayer = proxy;
                                         }));
      }
    },
    _disconnectPlayer: function() {
      if (this._playerName){
        this._disconnectSignals();
        global.log("Disconnecting Player: " + this._playerName)
        this._mediaServerPlayer = null
        this._playerName = null;
      }
    },

    _disconnectSignals: function() {
      global.log("Disconnecting Signals")
      this._proxy.ReleaseMediaPlayerKeysRemote("MediaKeystoMPRIS2");
      if (this._mediaKeyId)
        this._proxy.disconnectSignal(this._mediaKeyId);
        this._mediaKeyId = null
    },

    _onGrab: function() {

      
      

    },

    _onMediaKey: function(_emitter, _handleId, params) {
        let [_app, _method] = params
        global.log("Received key event: "+ _method);

        switch(_method){
          case "Play":
            global.log("Playing/Pausing Player");
            this._mediaServerPlayer.PlayPauseRemote();
            break;
          case "Stop":
            global.log("Stopping Player");
            this._mediaServerPlayer.StopRemote();
            break;
          case "Previous":
            global.log("Previous Track");
            this._mediaServerPlayer.PreviousRemote();
            break;
          case "Next":
            global.log("Next Track");
            this._mediaServerPlayer.NextRemote();
            break;
        }
        
        //
    },

    enable: function() {
      this._dbus = new DBusIface.DBus();
      this._proxy = new DBusIface.MediaKeys();
      this._playerName = null;
      let name_regex = /^org\.mpris\.MediaPlayer2\./;
      // load players
      this._dbus.ListNamesRemote(Lang.bind(this,
          function(names) {
              for (let n in names[0]) {
                  let name = names[0][n];
                  if (name_regex.test(name)) {
                      this._dbus.GetNameOwnerRemote(name, Lang.bind(this,
                          function(owner) {
                              if (!this._disabling){
                                this._playerName = name;
                                this._connectPlayer();
                                global.log("Connected Media Keys");
                              }
                          }
                      ));
                  }
              }
          }
      ));

      this._ownerChangedId = this._dbus.connectSignal('NameOwnerChanged', Lang.bind(this,
          function(proxy, sender, [name, old_owner, new_owner]) {
              if (name_regex.test(name)) {
                  if (!this._disabling) {
                      if (new_owner && !old_owner){
                          if (this._playerName != name){
                            this._playerName = name;
                            this._connectPlayer();
                            global.log("Reconnected Media Keys");
                          }
                      } else if (old_owner && !new_owner){
                          this._disconnectPlayer();
                          global.log("Deassigned Media Keys");
                      }
                      
                  }
              }
          }
      ));
    },

    disable: function() {
       this._disconnectPlayer();
    }
}


function init() {
    return new MediaKeysClient();
}